Package: mobileRNA
Type: Package
Title: mobileRNA: Investigate the RNA mobilome & population-scale
        changes
Version: 1.8.0
Authors@R: c(person("Katie", "Jeynes-Cupper", 
    email = "kejc@illinois.edu", role = c("aut", "cre"), 
     comment = c(ORCID = "0009-0000-1350-1371")),
    person("Marco", "Catoni", email = "m.catoni@bham.ac.uk", role = "aut", 
    comment= c(ORCID = "0000-0002-3258-2522")))
Description: Genomic analysis can be utilised to identify differences between RNA 
  populations in two conditions, both in production and abundance. This includes 
  the identification of RNAs produced by multiple genomes within a biological 
  system. For example, RNA produced by pathogens within a host or mobile RNAs in
  plant graft systems. The mobileRNA package provides methods to pre-process, 
  analyse and visualise the sRNA and mRNA populations based on the premise of
  mapping reads to all genotypes at the same time. 
License: MIT + file LICENSE
Depends: R (>= 4.3.0)
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: FALSE
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, BiocStyle, testthat (>= 3.0.0)
Imports: dplyr, tidyr, ggplot2, BiocGenerics, DESeq2, edgeR, ggrepel,
        grDevices, pheatmap, utils, tidyselect, progress, RColorBrewer,
        GenomicRanges, rtracklayer, data.table, SimDesign, scales,
        IRanges, stats, methods, Biostrings, reticulate, S4Vectors,
        GenomeInfoDb, SummarizedExperiment, rlang, bioseq, grid
biocViews: Visualization, RNASeq, Sequencing, SmallRNA, GenomeAssembly,
        Clustering, ExperimentalDesign, QualityControl, WorkflowStep,
        Alignment, Preprocessing
BiocType: Software
BugReports: https://github.com/KJeynesCupper/mobileRNA/issues
SystemRequirements: GNU make, ShortStack (>= 4.0), HTSeq, HISAT2,
        SAMtools, Conda
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/mobileRNA
git_branch: RELEASE_3_23
git_last_commit: 8240325
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:21:07 UTC; biocbuild
Author: Katie Jeynes-Cupper [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-1350-1371>),
  Marco Catoni [aut] (ORCID: <https://orcid.org/0000-0002-3258-2522>)
Maintainer: Katie Jeynes-Cupper <kejc@illinois.edu>
