- reduced_chr12_Eggplant.gff was obtained by downloading 
  Eggplant_V4.1_function_IPR_final.gff from 
  https://solgenomics.net/ftp/genomes/Solanum_melongena_V4.1/ in Octobter 2023,
  retaining only a few genomic features from chromosome 12 and 
  renaming the file.
  
- reduced_chr12_Eggplant.fa was obtained by downloading 
  Eggplant_V4.1.fa from 
  https://solgenomics.net/ftp/genomes/Solanum_melongena_V4.1/ in Octobter 2023,
  retaining only a section of the nucleotide sequence from chromosome 12 and 
  renaming the file.

- reduced_chr2_Tomato.gff was obtained by downloading 
  ITAG4.1_gene_models.gff from 
  https://solgenomics.net/ftp/tomato_genome/Heinz1706/annotation/ITAG4.1_release/
  in Octobter 2023,
  retaining only a few genomic features from chromosome 12 and 
  renaming the file.
  
- prefix_reduced_chr2_Tomato.gff was obtained by downloading 
  ITAG4.1_gene_models.gff from 
  https://solgenomics.net/ftp/tomato_genome/Heinz1706/annotation/ITAG4.1_release/
  in Octobter 2023,
  retaining only a few genomic features from chromosome 12 and 
  renaming the file. Contains "B_" prefix to chromosome names. 
  
- reduced_chr2_Tomato.fa was obtained by downloading 
  S_lycopersicum_chromosomes.4.00.fa from 
  https://solgenomics.net/ftp/tomato_genome/Heinz1706/assembly/build_4.00/
  in Octobter 2023, retaining only a section of the nucleotide sequence from 
  chromosome 12 and renaming the file.
  
  
- /mRNAseq/selfgraft_mRNAdemo_1.fq is a demo mRNAseq FASTQ file for a self-graft control 
  Eggplant (Solanum melongena). The original biological data file was taken from 
  the study Villanueva, G., Vilanova, S., Plazas, M., Prohens, J., & Gramazio, 
  P. (2023). Transcriptome profiles of eggplant (Solanum melongena) and its wild
  relative S. dasyphyllum under different levels of osmotic stress provide 
  insights into response mechanisms to drought. Current Plant Biology, 33, 
  100276. Accession number SRR10060529. This file was reduced in size to support 
  the workflow. 

- /mRNAseq/selfgraft_mRNAdemo_2.fq is a demo mRNAseq FASTQ file for a self-graft control 
  Eggplant (Solanum melongena). The original biological data file was taken from 
  the study Villanueva, G., Vilanova, S., Plazas, M., Prohens, J., & Gramazio, 
  P. (2023). Transcriptome profiles of eggplant (Solanum melongena) and its wild 
  relative S. dasyphyllum under different levels of osmotic stress provide 
  insights into response mechanisms to drought. Current Plant Biology, 33, 
  100276. Accession number SRR10060530. This file was reduced in size to support 
  the workflow. 

- /mRNAseq/heterograft_mRNAdemo_1.fq is a demo simulated mRNAseq FASTQ file for a 
  heterograft, simulating an Eggplant (Solanum melongena) scion and Tomato 
  (Solanum lycopersicum) rootstock. The original bioloigcal data for Eggplant 
  was taken from the study Villanueva, G., Vilanova, S., Plazas, M., Prohens, 
  J., & Gramazio, P. (2023). Transcriptome profiles of eggplant 
  (Solanum melongena) and its wild relative S. dasyphyllum 
  under different levels of osmotic stress provide insights into response 
  mechanisms to drought. Current Plant Biology, 33, 100276. Accession 
  number SRR10060529. The biological data for Tomato was taken from Tomato 
  Genome Consortium, X. (2012). The tomato genome sequence provides insights 
  into fleshy fruit evolution. Nature, 485(7400), 635. Accession number 
  SRR404309. This file was reduced in size to support the workflow. 

- /mRNAseq/heterograft_mRNAdemo_2.fq is a demo simulated mRNAseq FASTQ file for a 
  heterograft, simulating an Eggplant (Solanum melongena) scion and Tomato 
  (Solanum lycopersicum) rootstock. The original biological data for Eggplant 
  was taken from the study Villanueva, G., Vilanova, S., Plazas, M., Prohens, 
  J., & Gramazio, P. (2023). Transcriptome profiles of eggplant 
  (Solanum melongena) and its wild relative S. dasyphyllum 
  under different levels of osmotic stress provide insights into response 
  mechanisms to drought. Current Plant Biology, 33, 100276. Accession 
  number SRR10060530. The biological data for Tomato was taken from Tomato 
  Genome Consortium, X. (2012). The tomato genome sequence provides insights 
  into fleshy fruit evolution. Nature, 485(7400), 635. Accession number 
  SRR404309. This file was reduced in size to support the workflow. 


- /sRNAseq/selfgraft_demo_1.fq is a demo sRNAseq FASTQ file for a self-graft control 
  Eggplant (Solanum melongena). The original biological data file was taken
  from the study Li, B., Zhang, J., Gao, X., Pan, X., Zhou, R., & Wu, Y. (2022).
  Identification and Functional Analysis of MicroRNAs and Their Target Genes in 
  Reverse Thermosensitive Genic Male Sterility of Eggplant. Journal of the 
  American Society for Horticultural Science, 147(5), 249-259. Accession 
  number SRR17301403. This file was reduced in size to support the workflow. 

- /sRNAseq/selfgraft_demo_2.fq is a demo sRNAseq FASTQ file for a self-graft control 
  Eggplant (Solanum melongena). The original biological data file was taken
  from the study Li, B., Zhang, J., Gao, X., Pan, X., Zhou, R., & Wu, Y. (2022).
  Identification and Functional Analysis of MicroRNAs and Their Target Genes in 
  Reverse Thermosensitive Genic Male Sterility of Eggplant. Journal of the 
  American Society for Horticultural Science, 147(5), 249-259. Accession 
  number SRR17312491. This file was reduced in size to support the workflow. 

- /sRNAseq/selfgraft_demo_3.fq is a demo sRNAseq FASTQ file for a self-graft control 
  Eggplant (Solanum melongena). The original biological data file was taken
  from the study Li, B., Zhang, J., Gao, X., Pan, X., Zhou, R., & Wu, Y. (2022).
  Identification and Functional Analysis of MicroRNAs and Their Target Genes in 
  Reverse Thermosensitive Genic Male Sterility of Eggplant. Journal of the 
  American Society for Horticultural Science, 147(5), 249-259. Accession 
  number SRR17312490. This file was reduced in size to support the workflow. 


- /sRNAseq/heterograft_demo_1.fq is a demo simulated sRNAseq FASTQ file for a 
  heterograft, similuating an Eggplant (Solanum melongena) scion and Tomato 
  (Solanum lycopersicum) rootstock. The original biological data for Eggplant 
  was taken from the study Li, B., Zhang, J., Gao, X., Pan, X., Zhou, R., & Wu, 
  Y. (2022). Identification and Functional Analysis of MicroRNAs and Their 
  Target Genes in Reverse Thermosensitive Genic Male Sterility of Eggplant. 
  Journal of the American Society for Horticultural Science, 147(5), 249-259. 
  Accession number SRR17301403. The biological data for Tomato was taken from 
  Qing, Y., Zheng, Y., Mlotshwa, S., Smith, H. N., Wang, X., Zhai, X., ... & 
  Fei, Z. (2022). Dynamically expressed small RNAs, substantially driven by 
  genomic structural variants, contribute to transcriptomic changes during 
  tomato domestication. The Plant Journal, 110(6), 1536-1550. Accession number 
  SRR16077825. This file was reduced in size to support the workflow. 
  
- /sRNAseq/heterograft_demo_2.fq is a demo simulated sRNAseq FASTQ file for a 
  heterograft, similuating an Eggplant (Solanum melongena) scion and Tomato 
  (Solanum lycopersicum) rootstock. The original biological data for Eggplant 
  was taken from the study Li, B., Zhang, J., Gao, X., Pan, X., Zhou, R., & Wu, 
  Y. (2022). Identification and Functional Analysis of MicroRNAs and Their 
  Target Genes in Reverse Thermosensitive Genic Male Sterility of Eggplant. 
  Journal of the American Society for Horticultural Science, 147(5), 249-259. 
  Accession number SRR17312491. The biological data for Tomato was taken from 
  Qing, Y., Zheng, Y., Mlotshwa, S., Smith, H. N., Wang, X., Zhai, X., ... & 
  Fei, Z. (2022). Dynamically expressed small RNAs, substantially driven by 
  genomic structural variants, contribute to transcriptomic changes during 
  tomato domestication. The Plant Journal, 110(6), 1536-1550. Accession number 
  SRR16077825. This file was reduced in size to support the workflow. 


- /sRNAseq/heterograft_demo_3.fq is a demo simulated sRNAseq FASTQ file for a 
  heterograft, similuating an Eggplant (Solanum melongena) scion and Tomato 
  (Solanum lycopersicum) rootstock. The original biological data for Eggplant 
  was taken from the study Li, B., Zhang, J., Gao, X., Pan, X., Zhou, R., & Wu, 
  Y. (2022). Identification and Functional Analysis of MicroRNAs and Their 
  Target Genes in Reverse Thermosensitive Genic Male Sterility of Eggplant. 
  Journal of the American Society for Horticultural Science, 147(5), 249-259. 
  Accession number SRR17312490. The biological data for Tomato was taken from 
  Qing, Y., Zheng, Y., Mlotshwa, S., Smith, H. N., Wang, X., Zhai, X., ... & 
  Fei, Z. (2022). Dynamically expressed small RNAs, substantially driven by 
  genomic structural variants, contribute to transcriptomic changes during 
  tomato domestication. The Plant Journal, 110(6), 1536-1550. Accession number 
  SRR16077825. This file was reduced in size to support the workflow. 

