% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mobileRNA-package.R
\docType{data}
\name{sRNA_data}
\alias{sRNA_data}
\title{sRNA_data: simulated data for biological replicates}
\usage{
data(sRNA_data)
}
\value{
Dataframe in global environment
}
\description{
Simulated sRNAseq dataset
}
\details{
Simulates data is taken from eggplant and tomato sRNAseq samples and
created to simulate to movement of sRNA molecules from an Tomato rootstock to
an Eggplant Scion. Three Eggplant replicates were spiked with the same 150
tomato sRNA clusters, and named "heterograft_" 1 to 3. The analysis compares
these heterografts to three Eggplant self-grafts which are the original
un-spiked Eggplant replicates, called "selfgraft_" 1 to 3.

This data was imported and organised by the \code{\link[=RNAimport]{RNAimport()}} function.
}
\examples{
 data("sRNA_data")
}
